#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, io, json, sys, subprocess, webbrowser
# Python 3
try:
    from tkinter import *
    from tkinter.ttk import *
    import tkinter.messagebox as messagebox
    import tkinter.filedialog as filedialog
# Python 2
except:
    from Tkinter import *
    from ttk import *
    import tkMessageBox as messagebox
    import tkFileDialog as filedialog

import StatusBar, ToolBar
import Korpus, LangOpt
try:
    from win32com.client import Dispatch
except:
    pass


######################
# Konfigurieren...

# SkriptDir = os.path.dirname(os.path.abspath(__file__))
# Dies scheint besser zu funktionieren:
SkriptDir = os.path.dirname(sys.argv[0])
ImgDir = os.path.join(SkriptDir, 'Images')
cfgFile = os.path.join(SkriptDir, 'TextSTAT.ini')
# cfgFile = os.path.join(os.path.expanduser("~"), '.textstat3.ini')

try:
    with open(cfgFile) as infile:
        cfg = json.load(infile)
except:
    cfg = {}
    cfg['Benutzersprache'] = 'en'
    cfg['KorpDir'] = os.path.expanduser("~")
    cfg['FileDir'] = os.path.expanduser("~")
    cfg['LetzteKorpora'] = []
    cfg['freqMin'] = '0'
    cfg['freqMax'] = '0'
    cfg['konkContextL'] = '40'
    cfg['konkContextR'] = '40'
    cfg['konkMarkL'] = '{'
    cfg['konkMarkR'] = '}'    
    cfg['zitContextL'] = '500'
    cfg['zitContextR'] = '500'
    cfg['FileKodierung'] = 'utf_8'
    cfg['fenster'] = '800x600+10+10'    # d.i.: Breite x Höhe + xOffset + yOffset


# Sprache
if cfg['Benutzersprache'] == 'en':
    s = LangOpt.english
elif cfg['Benutzersprache'] == 'nl':
    s = LangOpt.nederlands
elif cfg['Benutzersprache'] == 'pt':
    s = LangOpt.portuguese
elif cfg['Benutzersprache'] == 'es':
    s = LangOpt.espanol
elif cfg['Benutzersprache'] == 'fr':
    s = LangOpt.francais
elif cfg['Benutzersprache'] == 'gl':
    s = LangOpt.galician
elif cfg['Benutzersprache'] == 'fi':
    s = LangOpt.suomi
elif cfg['Benutzersprache'] == 'po':
    s = LangOpt.polski
elif cfg['Benutzersprache'] == 'cs':
    s = LangOpt.czech
elif cfg['Benutzersprache'] == 'it':
    s = LangOpt.italiano
elif cfg['Benutzersprache'] == 'ca':
    s = LangOpt.catalan
elif cfg['Benutzersprache'] == 'tr':
    s = LangOpt.turkish
else:
    s = LangOpt.deutsch


###############
# GUI

root = Tk()
root.title(s['Titel_lang'])
root.geometry(cfg['fenster'])
root.minsize(650, 550)   # (Breite, Höhe)


#root.option_add('*Text*background', 'white')
#root.option_add('*Listbox*background', 'white')
if sys.platform == 'win32':
    ico = os.path.join(SkriptDir, 'Images/TextSTAT.ico')
    root.iconbitmap(bitmap=ico)
    root.option_add('*Dialog.msg*font', 'Sans 9')
    root.option_add('*Text*font', 'Sans 10')
    Rechtsklick = "<Button-3>"
elif sys.platform == "darwin":
    root.option_add('*Dialog.msg*font', 'Sans 12')
    root.option_add('*Text*font', 'Sans 12')
    # Muss betriebssystemabhängig definiert werden, da Mac sich hier anders verhält
    Rechtsklick = "<Button-2>"
else:
    root.option_add('*Dialog.msg*font', 'Sans 9')
    root.option_add('*Text*font', 'Sans 10')
    Rechtsklick = "<Button-3>"


# Styling; see:
# http://infohost.nmt.edu/tcc/help/pubs/tkinter/web/ttk-style-layer.html
# http://www.tkdocs.com/tutorial/styles.html
# Das müsste dann noch wohl für jedes Betriebssystem einzeln angegeben werden.. 
# sty = Style()
# sty.configure('Treeview', font = "Mono 9")


class GUI:
    def __init__(self, root):
        self.root = root
        self.korpus = None

        # Wenn Fenster über Windows-Fenstermanager geschlossen wird...
        self.root.protocol( 'WM_DELETE_WINDOW', self.ende )
        
        self.fenster = Frame(self.root)
        self.fenster.pack(expand=YES, fill=BOTH )

        # Das Menu
        MeinMenu = Menu(self.root)
        self.root.config(menu=MeinMenu)

        self.filemenu = Menu(MeinMenu, tearoff=0)
        MeinMenu.add_cascade(label=s['Korpus'], underline=0, menu = self.filemenu)
        self.filemenu.add_command(label=s['KorpusNew'], underline=0, command = self.korpus_new)
        self.filemenu.add_command(label=s['KorpusOpen'], underline=0, command = self.korpus_open)
        self.recentfilesmenu = Menu(self.filemenu, tearoff=0)
        self.filemenu.add_cascade(label=s['LetzteKorpora'], underline=0, menu=self.recentfilesmenu)
        self.update_recentfiles_menu()
        
        self.filemenu.add_separator()
        self.filemenu.add_command(label=s['KorpusSave'], underline=2, command = self.korpus_save)        
        self.filemenu.add_separator()
        self.filemenu.add_command(label=s['DateiHinzu'], underline=0, command = self.datei_hinzu)
        self.filemenu.add_command(label=s['WebHinzu'], underline=0, command = self.WebDialog)
        self.filemenu.add_command(label=s['KorpusHinzu'], underline=1, command = self.korpus_import)
        self.filemenu.add_separator()
        self.filemenu.add_command(label=s['ImpKorpus'], underline=7, command = self.korpus_import)
        self.filemenu.add_separator()
        self.filemenu.add_command(label=s['DateiWeg'], underline=2, command = self.datei_entfernen)
        self.filemenu.add_separator()
        self.filemenu.add_command(label=s['Exit'], underline=1, command = self.ende)

        exportmenu = Menu(MeinMenu, tearoff=0)
        MeinMenu.add_cascade(label=s['ImpEx'], underline=0, menu = exportmenu)
        exportmenu.add_command(label=s['Ex2csv'], underline=0, command = self.exportFreq2csv)
        exportmenu.add_command(label=s['ExConc2csv'], underline=0, command = self.exportConc2csv)
        exportmenu.add_command(label=s['ExConc2txt'], underline=0, command = self.exportConc2txt)
        if os.name == 'nt':
            exportmenu.add_separator()
            exportmenu.add_command(label=s['ExFreq2excel'], underline=1, command = self.exportFreq2excel)
            exportmenu.add_command(label=s['ExConc2excel'], underline=1, command = self.exportConc2excel)


        codepagemenu = Menu(MeinMenu, tearoff=1)
        MeinMenu.add_cascade(label=s['CodePage'], underline=1, menu = codepagemenu)
        self.dateikodierung = StringVar()
        self.dateikodierung.set(cfg['FileKodierung'])
        kodierungen = [
            ['UTF-8 (Unicode)', 'utf_8'],
            ['UTF-16 (Unicode)', 'utf_16'],
            ['ISO-8859-1 (Latin-1)', 'iso8859_1'],
            ['ISO-8859-2 (Latin-2)', 'iso8859_2'],
            ['ISO-8859-5 (Cyrillic)', 'iso8859_5'],
            ['ISO-8859-6 (Arabic)', 'iso8859_6'],
            ['ISO-8859-7 (Greek)', 'iso8859_7'],
            ['ISO-8859-8 (Hebrew)', 'iso8859_8'],
            ['ISO-8859-9 (Turkish)', 'iso8859_9'],
            ['CP1252 (ANSI, WinLatin-1)', 'cp1252'],
            ['CP1250 (WinLatin-2)', 'cp1250'],
            ['CP1251 (WinCyrillic)', 'cp1251'],
            ['CP1254 (WinTurkish)', 'cp1254'],
            ['CP1255 (WinHebrew)', 'cp1255'],
            ['CP1256 (WinArabic)', 'cp1256'],
            ['CP850 (MS DOS)', 'cp850'],
            ['KOI 8 R (Russian)', 'koi8_r'],
            ['GB 2312 (Simplified Chinese)', 'gb2312'],
            ['BIG 5 (Traditional Chinese)', 'big5'],
            ]
        for bezeichnung, kodierung in kodierungen:
            codepagemenu.add_radiobutton(label=bezeichnung, variable=self.dateikodierung, value=kodierung)  


        sprachmenu = Menu(MeinMenu, tearoff=0)
        MeinMenu.add_cascade(label=s['Sprache'], underline=0, menu = sprachmenu)
        self.sprache_aendern = StringVar()
        self.sprache_aendern.set(cfg['Benutzersprache'])
        oberflaechensprachen = [
            ['Deutsch', 'de'],
            ['English', 'en'],
            ['Nederlands', 'nl'],
            ['Español', 'es'],
            ['Português', 'pt'],
            ['Français', 'fr'],
            ['Italiano', 'it'],
            ['Català', 'ca'],
            ['Galego', 'gl'],
            ['Suomi', 'fi'],
            ['Polski', 'po'],
            ['Čeština', 'cs'],
            ['Türkçe', 'tr'],
            ]
        for sprach_label, sprach_abkz in oberflaechensprachen:
            sprachmenu.add_radiobutton(label=sprach_label, underline=0, variable=self.sprache_aendern, value=sprach_abkz, command=self.sprachwarnung)
            

        # Hilfe- / Info-Menu
        hilfemenu = Menu(MeinMenu, tearoff=0)
        MeinMenu.add_cascade(label=s['Hilfe'], underline=1, menu = hilfemenu)
        hilfemenu.add_command(label=s['RegExpHelp'], underline=0, command = self.regex_hilfe)
        hilfemenu.add_separator()
        hilfemenu.add_command(label=s['History'], underline=0, command = self.zeigHistory)
        hilfemenu.add_command(label=s['License'], underline=0, command = self.zeigLicense)
        hilfemenu.add_separator()
        hilfemenu.add_command(label=s['Info'], underline=0, command = self.about)

        # Die Statuszeile
        self.status = StatusBar.StatusBar(self.root)
        self.status.pack(fill=X)
        self.status.set(s['Willkommen'])
        self.status.indicator(s['StatusNoKorp'])

        # Der Toolbar
        # Die Icons stammen von http://adamwhitcroft.com/batch/
        self.toolbar = ToolBar.ToolBar(self.fenster, statusbar = self.status)
        self.toolbar.pack(side=TOP, fill=X, pady=3)
        self.toolbar.add('NeuButton', s['KorpusNew'], ImgDir+'/database.gif', self.korpus_new, s['KorpusNew2'])
        self.toolbar.add('KorpusOeffnenButton', s['KorpusOpen'], ImgDir+'/folder.gif', self.korpus_open, s['KorpusOpen2'])
        self.toolbar.add('SaveButton', s['KorpusSave'], ImgDir+'/floppy.gif', self.korpus_save, s['KorpusSave2'])
        self.toolbar.add_separator()
        self.toolbar.add('KorpusHinzuButton', s['KorpusHinzu'], ImgDir+'/folder-add.gif', self.korpus_import, s['KorpusHinzu2'])
        self.toolbar.add('HTMLhinzuButton', s['WebHinzu'], ImgDir+'/browser-download.gif', self.WebDialog, s['WebHinzu2'] )
        self.toolbar.add('HinzuButton', s['DateiHinzu'], ImgDir+'/in.gif', self.datei_hinzu, s['DateiHinzu2'] )
        self.toolbar.add('EntfernButton', s['DateiWeg'], ImgDir+'/out.gif', self.datei_entfernen, s['DateiWeg2'])
        self.toolbar.add_separator()
        self.toolbar.add('FrequenzButton', s['FreqZeigen'], ImgDir+'/list.gif', self.zeigWortfrequenz, s['FreqZeigen2'])
        self.toolbar.add_separator()
        self.toolbar.add('CopyButton', s['Kopieren'], ImgDir+'/clipboard.gif', self.copy2clipboard, s['Kopieren2'])
        self.toolbar.add_separator()
        self.toolbar.add('HelpButton', s['Info'], ImgDir+'/bulb-2.gif', self.about, s['Info2'])


        # Die Arbeitsflaeche
        self.nb = Notebook(self.fenster, name='notebook')
        self.nb.enable_traversal()
        self.nb.pack(fill=BOTH, expand=TRUE, padx=2, pady=3)
        self.machKorpusTab()
        self.machFormenTab()
        self.machKonkordanzTab()
        self.machZitatTab()


        # Beim ersten Start: Sprache festlegen
        if not os.path.exists(cfgFile):
            self.optionen_speichern()
            # Auf dem Mac hängt das Programm sonst...
            if sys.platform != "darwin":
                messagebox.showinfo('TextSTAT language', 'Welcome to TextSTAT! \n\nYou can change the language of the TextSTAT interface (Menu > Language) \n\nAvailable Languages are: English, German, Dutch, French, Italian, Potuguese, Spanish, Galician, Catalan, Finnish, Polish, Czech, and Turkish')

            
    def machKorpusTab(self):
        frame = Frame(self.nb, name='korpustab')

        self.infotxt = StringVar()
        self.infolabel = Label(frame, wraplength=200, justify=LEFT, textvariable=self.infotxt)
        self.infolabel.pack(side=RIGHT, padx=10, pady=10)
        self.infotxt.set(s['KTinfo'])

        listframe = Frame(frame)
        listframe.pack(expand=YES, fill=BOTH, side=LEFT, padx=5, pady=5)

        self.datliste = Treeview(listframe, columns=('dat',), show="headings" )
        self.datliste.pack(side=LEFT, fill=BOTH, expand=YES)
        vsb = Scrollbar(listframe, orient="vertical", command=self.datliste.yview)
        vsb.pack(side=RIGHT, fill=Y)
        self.datliste.configure(yscrollcommand=vsb.set)

        self.datliste.heading('dat', text=s['Korpus'])
                
        self.datliste.bind("<Double-1>", self.datei_anzeigen)
        self.datliste.bind("<Enter>", self.statusAktualisieren)
        self.datliste.bind(Rechtsklick, self.KorpusTabPopup)      
        
        self.nb.add(frame, text=s['KorpusTab'], underline=0, padding=2)


    def KorpusTabPopup(self, event):
        self.KorpusPopupMenu = Menu(self.root, tearoff=0)
        self.KorpusPopupMenu.add_command(label=s['KTdatinfo'], command=self.datei_info)
        self.KorpusPopupMenu.add_command(label=s['KTedit'], command=self.datei_anzeigen)
        self.KorpusPopupMenu.add_command(label=s['KTopen'], command=self.datei_oeffnen)
        self.KorpusPopupMenu.add_command(label=s['KTkopieren'], command=self.copy2clipboard)
        self.KorpusPopupMenu.add_command(label=s['KTdatweg'], command=self.datei_entfernen)
        self.KorpusPopupMenu.tk_popup(*self.root.winfo_pointerxy())


    def machFormenTab(self):
        frame = Frame(self.nb, name='formentab')
        listframe = Frame(frame)
        listframe.pack(side=LEFT, expand=YES, fill=BOTH, padx=5, pady=5)
        
        self.freqliste = Treeview(listframe, columns=('wort', 'freq'), show="headings" )
        vsb = Scrollbar(listframe, orient="vertical", command=self.freqliste.yview)
        vsb.pack(side=RIGHT, fill=Y)
        self.freqliste.configure(yscrollcommand=vsb.set)
        self.freqliste.pack(side=LEFT, fill=BOTH, expand=YES)
        self.freqliste.heading('wort', text=s['FTwortform'])
        self.freqliste.heading('freq', text=s['FTfrequenz'])
                
        self.freqliste.bind("<Double-1>", self.zeigKonkordanz)
        self.freqliste.bind("<Return>", self.zeigKonkordanz)
        self.freqliste.bind("<Enter>", self.statusAktualisieren)
        self.freqliste.bind(Rechtsklick, self.FormenTabPopup)

        optFrame = LabelFrame(frame, text=s['FTOptionen'])
        optFrame.pack(fill=X, padx=5, pady=5)
        # Label(optFrame, text=s['FTinfo'], justify=LEFT).pack(anchor=W)

        optFrame1 = Frame(optFrame)
        optFrame1.pack(fill=BOTH, padx=5, pady=10)
        self.sortModusFreq = StringVar()
        Radiobutton(optFrame1, text=s['FTsortFreq'], value='freq', variable=self.sortModusFreq).pack(anchor=W)
        Radiobutton(optFrame1, text=s['FTsortAlpha'], value='alpha', variable=self.sortModusFreq).pack(anchor=W)
        Radiobutton(optFrame1, text=s['FTsortRetro'], value='retro', variable=self.sortModusFreq).pack(anchor=W)
        self.sortModusFreq.set('freq')

        optFrame2 = Frame(optFrame)
        optFrame2.pack(fill=X, padx=5)
        self.minFreq = StringVar()
        self.minFreq.set(cfg['freqMin'])
        Spinbox(optFrame2, width=5, textvariable=self.minFreq, values=('---',1,2,3,4,5,10,25,50,100,250,500,750,1000,2500,5000,10000)).pack(side=LEFT, anchor=W)
        Label(optFrame2, text=s['FTminFreq']).pack(side=LEFT, padx=5, pady=1)

        optFrame3 = Frame(optFrame)
        optFrame3.pack(fill=X, padx=5, pady=2)
        self.maxFreq = StringVar()
        self.maxFreq.set(cfg['freqMax'])
        Spinbox(optFrame3, width=5, textvariable=self.maxFreq, values=('---',1,2,3,4,5,10,25,50,100,250,500,750,1000,2500,5000,10000)).pack(side=LEFT, anchor=W)
        Label(optFrame3, text=s['FTmaxFreq']).pack(side=LEFT, padx=5, pady=1)

        self.grosskleinFreq = IntVar()
        Checkbutton(optFrame, text=s['FTignGross'], variable=self.grosskleinFreq).pack(padx=5, pady=10, anchor=W)

        self.suchFreq = StringVar()
        Label(optFrame, text=s['FTsuchFreq'], justify=LEFT).pack(anchor=W, padx=5, pady=10)
        self.suchFreqEntry = Entry(optFrame, textvariable=self.suchFreq)
        self.suchFreqEntry.pack(anchor=W, padx=5)
        self.suchFreqEntry.bind('<Return>', self.zeigWortfrequenz)

        Button(optFrame, text=s['FTfreqliste'], command=self.zeigWortfrequenz).pack(anchor=W, padx=5, pady=10)

        self.nb.add(frame, text=s['FormenTab'], underline=0, padding=2)


    def FormenTabPopup(self, event):
        self.FormenPopupMenu = Menu(self.root, tearoff=0)
        self.FormenPopupMenu.add_command(label=s['FTkopieren'], command=self.copy2clipboard)
        self.FormenPopupMenu.add_command(label=s['FTsuchKonk'], command=self.zeigKonkordanz)
        self.FormenPopupMenu.add_command(label=s['FTzeigInfo'], command=self.zeigFormInfo)
        self.FormenPopupMenu.tk_popup(*self.root.winfo_pointerxy())


    def machKonkordanzTab(self):
        frame = Frame(self.nb, name='conctab')
        suchFrame = Frame(frame)
        suchFrame.pack(fill=X, padx=10, pady=5)
        self.suchwort = Entry(suchFrame, width=40, takefocus=1)
        self.suchwort.pack(side=LEFT)
        self.suchwort.bind("<Return>", self.zeigKonkordanz)
        Button(suchFrame, text=s['CTsuchen'], takefocus=1, command=self.zeigKonkordanz).pack(side=LEFT, padx=5, pady=10)
        Button(suchFrame, text=s['CTmachQuery'], command=self.QueryDialog).pack(anchor=W, padx=20, pady=10)

        listframe = Frame(frame)
        listframe.pack(side=LEFT, expand=YES, fill=BOTH, padx=5, pady=5)
        
        self.konkliste = Treeview(listframe, columns=('konk',), show="headings" )
        vsb = Scrollbar(listframe, orient="vertical", command=self.konkliste.yview)
        vsb.pack(side=RIGHT, fill=Y)
        self.konkliste.configure(yscrollcommand=vsb.set)
        self.konkliste.pack(side=LEFT, fill=BOTH, expand=YES)
        self.konkliste.heading('konk', text=s['ConcTab'])
                
        self.konkliste.bind("<Double-1>", self.zeigZitat)
        self.konkliste.bind("<Enter>", self.statusAktualisieren)
        self.konkliste.bind(Rechtsklick, self.KonkTabPopup)
        
        optFrame = LabelFrame(frame, text=s['CTOptionen'])
        optFrame.pack(fill=X, padx=5, pady=5)

        optFrame1 = Frame(optFrame)
        optFrame1.pack(fill=BOTH, padx=5, pady=10)
        self.konkGanzewoerter = IntVar()
        Checkbutton(optFrame1, text=s['CTganzWort'], variable=self.konkGanzewoerter).pack(anchor=W)
        self.konkGrossklein = IntVar()
        self.konkGrosskleinCheckbutton = Checkbutton(optFrame1, text=s['CTignGross'], variable=self.konkGrossklein)
        self.konkGrosskleinCheckbutton.pack(anchor=W)
        self.markKonk = IntVar()
        self.markKonk.set(1)
        Checkbutton(optFrame1, text=s['CTmarkSuch'], variable=self.markKonk).pack(anchor=W)

        optFrame2 = Frame(optFrame)
        optFrame2.pack(fill=X, padx=5)
        self.konkContextL = StringVar()
        self.konkContextL.set(cfg['konkContextL'])
        Spinbox(optFrame2, width=4, textvariable=self.konkContextL, from_=10, to=100, increment=10).pack(side=LEFT, anchor=W)
        Label(optFrame2, text=s['CTcontextL']).pack(side=LEFT, padx=5, pady=1)

        optFrame3 = Frame(optFrame)
        optFrame3.pack(fill=BOTH, padx=5)
        self.konkContextR = StringVar()
        self.konkContextR.set(cfg['konkContextR'])
        Spinbox(optFrame3, width=4, textvariable=self.konkContextR, from_=10, to=100, increment=10).pack(side=LEFT, anchor=W)
        Label(optFrame3, text=s['CTcontextR']).pack(side=LEFT, padx=5, pady=1)

        optFrame4 = Frame(optFrame)
        optFrame4.pack(fill=BOTH, padx=5, pady=10)
        self.sortModusKonk = StringVar()
        Radiobutton(optFrame4, text=s['CTsortAlpha'], value='alpha', variable=self.sortModusKonk).pack(anchor=W)
        Radiobutton(optFrame4, text=s['CTsortR'], value='rechts', variable=self.sortModusKonk).pack(anchor=W)
        Radiobutton(optFrame4, text=s['CTsortL'], value='links', variable=self.sortModusKonk).pack(anchor=W)
        self.sortModusKonk.set('alpha')

        Button(optFrame, text=s['CTaktualisier'], command=self.zeigKonkordanz).pack(anchor=W, padx=5, pady=10)

        self.nb.add(frame, text=s['ConcTab'], underline=1, padding=2)


    def KonkTabPopup(self, event):
        self.KonkPopupMenu = Menu(self.root, tearoff=0)
        self.KonkPopupMenu.add_command(label=s['CTkopieren'], command=self.copy2clipboard)
        self.KonkPopupMenu.add_command(label=s['CTzeigZitat'], command=self.zeigZitat)
        self.KonkPopupMenu.tk_popup(*self.root.winfo_pointerxy())


    def machZitatTab(self):
        frame = Frame(self.nb, name='zitattab')
        Label(frame, text=s['ZTinfo']).pack(padx=10, pady=10)

        txtframe = Frame(frame)
        txtframe.pack(expand=YES, fill=BOTH, side=LEFT, padx=5, pady=5)
        scrollbar = Scrollbar(txtframe, orient=VERTICAL)
        self.zitat = Text(txtframe, wrap=WORD, padx=10, pady=1, yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.zitat.yview)
        scrollbar.pack(side=RIGHT, fill=Y)

        self.zitat.pack(side=LEFT, expand=YES, fill=BOTH)
        self.zitat.tag_configure('blau', foreground='blue', underline=1)
        self.zitat.tag_configure('rot', foreground='red')
        self.zitat.tag_bind('oeffnen', '<Double-Button-1>', self.zitatDatei_oeffnen)
        self.zitat.bind(Rechtsklick, self.ZitatTabPopup)
        self.zitat.config(cursor='arrow')
        self.zitat.insert(END, s['ZTInit'])

        self.nb.add(frame, text=s['ZitatTab'], underline=1, padding=2)

    def ZitatTabPopup(self, event):
        self.ZitatPopupMenu = Menu(self.root, tearoff=0)
        self.ZitatPopupMenu.add_command(label=s['ZTkopieren'], command=self.copy2clipboard)
        self.ZitatPopupMenu.add_command(label=s['ZToeffnen'], command=self.zitatDatei_oeffnen)
        self.ZitatPopupMenu.tk_popup(*self.root.winfo_pointerxy())


##### Dialoge #######

    def WebDialog(self):
        try:
            self.wd.focus_set()
        except:
            self.wd = Toplevel(self.root)
            self.wd.title("Web2corpus - TextSTAT WebSpider")
            frame = Frame(self.wd)
            frame.pack(expand=YES, fill=BOTH)
            Label(frame, justify=LEFT, text=s['WDinfo']).pack(anchor=W, padx=20, pady=10)
            self.webUrl = StringVar()
            url = Entry(frame, width=50, textvariable=self.webUrl)
            url.pack(anchor=W, padx=20, pady=5)
            Label(frame, text=s['WDfrageSeiten']).pack(anchor=W, padx=20, pady=5)

            frame1 = Frame(frame)
            frame1.pack(expand=YES, fill=BOTH)
            self.webAnzahl = StringVar()
            self.webAnzahl.set('1')
            Spinbox(frame1, width=5, textvariable=self.webAnzahl, values=(1,10,25,50,100,250,500,750,1000)).pack(side=LEFT, anchor=W, padx=20, pady=5)
            Label(frame1, text=s['WDanzahl']).pack(side=LEFT)
            self.webBasis = StringVar()
            self.webBasis.set('server')
            Radiobutton(frame, text=s['WDinSubdir'], value='subdir', variable=self.webBasis).pack(anchor=W, padx=20 )
            Radiobutton(frame, text=s['WDaufServer'], value='server', variable=self.webBasis).pack(anchor=W, padx=20 )
            Button(frame, text=s['WDok'], width=10, command=self.webFiles_hinzu).pack(side=LEFT, padx=20, pady=20)
            Button(frame, text=s['WDcancel'], width=10, command=self.wd.destroy).pack(side=RIGHT, padx=20, pady=20)
            url.focus_set()


    def QueryDialog(self):
        try:
            self.qd.focus_set()
        except:
            self.qd = Toplevel(self.root)
            self.qd.title("TextSTAT QueryEditor")
            frame = Frame(self.qd)
            frame.pack(expand=YES, fill=BOTH)
            Label(frame, justify=LEFT, text=s['QDinfo']).pack(anchor=W, padx=10, pady=15)

            frame1 = LabelFrame(frame, text=s['QDsuchFelder'])
            frame1.pack(fill=BOTH, expand=YES, padx=10)
            frame1a = Frame(frame1)
            frame1a.pack(expand=YES, fill=BOTH)
            self.qAusdruck1 = Entry(frame1a)
            self.qAusdruck1.pack(side=LEFT, anchor=W, padx=5, pady=5)
            Label(frame1a, text=s['QAusdruck1']).pack(side=LEFT, padx=5, pady=5)
            frame1b = Frame(frame1)
            frame1b.pack(expand=YES, fill=BOTH)
            self.qAusdruck2 = Entry(frame1b)
            self.qAusdruck2.pack(side=LEFT, anchor=W, padx=5, pady=5)
            Label(frame1b, text=s['QAusdruck2']).pack(side=LEFT, padx=5, pady=5)

            frame2 = LabelFrame(frame, text=s['QAbstand'])
            frame2.pack(fill=BOTH, expand=YES, padx=10, pady=25)
            self.qAbstandMin = Spinbox(frame2, width=5, from_=0, to=10)
            self.qAbstandMin.pack(side=LEFT, anchor=W, padx=5, pady=5)
            Label(frame2, text=s['QAbstandMin']).pack(side=LEFT, padx=5, pady=5)
            self.qAbstandMax = Spinbox(frame2, width=5, from_=0, to=10)
            self.qAbstandMax.pack(side=LEFT, anchor=W, padx=5, pady=5)
            Label(frame2, text=s['QAbstandMax']).pack(side=LEFT, padx=5, pady=5)

            Button(frame, text=s['QmachAusdruck'], command=self.query_machen).pack(side=LEFT, padx=20, pady=10)
            Button(frame, text=s['QDcancel'], width=10, command=self.qd.destroy).pack(side=RIGHT, padx=20, pady=10)
            self.qd.bind("<Return>", self.query_machen)
            self.qAusdruck1.focus_set()


    def query_machen(self, event=''):
        a1 = self.qAusdruck1.get()
        a1 = a1.replace('*', '\w*')
        a1 = a1.replace('?', '\w')
        a2 = self.qAusdruck2.get()
        a2 = a2.replace('*', '\w*')
        a2 = a2.replace('?', '\w')
        ab1 = self.qAbstandMin.get()
        ab2 = self.qAbstandMax.get()
        if a1 == '' and a2 == '':
            messagebox.showwarning(s['Error'], s['ErrQuery'])
            self.qAusdruck1.focus_set()
            return

        if ab1 == '0' and ab2 == '0':
            such = "%s\W+%s" % (a1, a2)
        else:
            such = "%s(\W+\w+){%s,%s}\W+%s" % (a1, ab1, ab2, a2)
        self.busyCursorOn()
        self.suchwort.delete(0, END)
        self.suchwort.insert(END, such)
        self.konkGrossklein.set(1)
        self.konkGanzewoerter.set(1)
        self.zeigKonkordanz()
        self.busyCursorOff()


    def TextDialog(self, text='', titel='TextSTAT'):
        td = Toplevel(self.root)
        td.transient(self.root)
        td.title(titel)
        # self.td.geometry('600x500+50+50')
        frame = Frame(td)
        frame.pack(expand=YES, fill=BOTH)

        scrollbar = Scrollbar(frame, orient=VERTICAL)
        txt = Text(frame, wrap=WORD, padx=10, pady=1, yscrollcommand=scrollbar.set)
        scrollbar.config(command=txt.yview)
        scrollbar.pack(side=RIGHT, fill=Y, padx=0)
        txt.pack(side=LEFT, expand=YES, fill=BOTH)
        Button(td, text=' OK ', command=td.destroy).pack(side=BOTTOM, pady=10)
        txt.insert(END, text)
        td.focus_set()


    def TextEditDialog(self, text='', titel='TextSTAT'):
        self.editFilename = titel
        self.ted = Toplevel(self.root)
        self.ted.transient(self.root)
        self.ted.title(titel)
        # self.ted.geometry('600x500+50+50')
        self.ted.minsize(500, 500)
        frame = Frame(self.ted)
        frame.pack(expand=YES, fill=BOTH)
        frame1 = Frame(self.ted)
        frame1.pack(fill=X)

        scrollbar = Scrollbar(frame, orient=VERTICAL)
        self.editTxt = Text(frame, wrap=WORD, padx=10, pady=1, yscrollcommand=scrollbar.set)
        scrollbar.config(command=self.editTxt.yview)
        scrollbar.pack(side=RIGHT, fill=Y, padx=0)
        self.editTxt.pack(side=LEFT, expand=YES, fill=BOTH)
        Button(frame1, text=s['TxtEditSave'], command=self.datei_speichern).pack(side=LEFT, padx=50, pady=20)
        Button(frame1, text=s['TxtEditClose'], command=self.ted.destroy).pack(side=RIGHT, padx=50, pady=20)
        self.editTxt.insert(END, text)
        self.ted.focus_set()


    def zeigHistory(self):
        f = os.path.join(SkriptDir, 'History.txt')
        text = open(f).read()
        t = s['Titel'] + ' - ' + s['History']
        self.TextDialog(text=text, titel=t)


    def zeigLicense(self):
        f = os.path.join(SkriptDir, 'License.txt')
        text = open(f).read()
        t = s['Titel'] + ' - ' + s['License']
        self.TextDialog(text=text, titel=t)


### Die eigentlichen Korpus-Funktionen ###

    def korpus_new(self):
        try:
            # Evtl. offene Datenbank erst schließen
            if self.korpus:
                self.korpus.db_close()
                
            f = filedialog.asksaveasfilename(initialdir=cfg['KorpDir'], defaultextension='.crp3', title=s['KorpusNew2'])
            if os.path.isfile(f):
                # Bestehende Datenbank löschen
                os.remove(f)

            # Default-Dir neu setzen
            cfg['KorpDir'] = os.path.dirname(f)
            self.korpus = Korpus.Korpus(self)
            self.korpus.db_new(f)
            self.status.set(self.korpus.getDbName())
            self.root.title(s['Titel'] + ' - ' + self.korpus.getDbName())
            self.zeigDateiliste()
            self.add_recent(filename=f)
            messagebox.showinfo(s['CodePage'], s['KodierungInfo'] % cfg['FileKodierung'])
        except:
            messagebox.showwarning(s['Error'], s['ErrKorpNew'])


    def korpus_open(self, f=''):
        # Wenn kein Dateipfad übergeben wird
        if f == '':
            f = filedialog.askopenfilename(initialdir=cfg['KorpDir'], title=s['KorpusOpen'])
            
        if os.path.isfile(f):
            if self.korpus:
                # Evtl. offene Datenbank erst schließen
                self.korpus.db_close()                  
            try:
                # Default-Dir neu setzen
                self.korpus = Korpus.Korpus(self) 
                self.korpus.db_open(f)
                cfg['KorpDir'] = os.path.dirname(f)
                self.korpusdatei = f
                self.status.set(self.korpus.getDbName())
                self.root.title(s['Titel'] + ' - ' + self.korpus.getDbName())
                self.zeigDateiliste()
                self.add_recent(filename=f)
            except:
                self.korpus = None
                self.root.title(s['Titel'])
                messagebox.showwarning(s['Error'], s['ErrKorpOpen'])


    def korpus_import(self):
        if self.korpus != None:
            try:
                f = filedialog.askopenfilename(initialdir=cfg['KorpDir'], title=s['KorpusHinzu'])
                # Default-Dir neu setzen
                cfg['KorpDir'] = os.path.dirname(f)
                self.korpus.db_import(f)
                self.status.set(s['StatusKorpImp'])
                self.zeigDateiliste()
            except Exception as e:
                # print(e)
                messagebox.showwarning(s['Error'], s['ErrKorpImp'])
        else:
            messagebox.showwarning(s['Error'], s['ErrNoKorp'])


    def korpus_save(self):
        self.optionen_speichern()
        if self.korpus != None:
            self.korpus.db_save()
            

    def datei_hinzu(self):
        if self.korpus != None:
            self.statusAktualisieren()
            filenames = filedialog.askopenfilename(initialdir=cfg['FileDir'], filetypes= (('All', '*.*'),('Text', '*.txt'),('HTML', '*.htm*'),('MS Word', '*.doc'),('MS Word', '*.docx'),('LibreOffice/OpenOffice', '*.odt')), multiple=1)

            # Default-Dir neu setzen
            if len(filenames) > 0:
                cfg['FileDir'] = os.path.dirname(filenames[0])
            
            # Progressbar einstellen
            self.status.progress.config(maximum = len(filenames))
            
            # Files einzeln einlesen
            for filename in filenames:
                try:
                    self.korpus.file_add(filename, encoding=self.dateikodierung.get())
                except:
                    meldung = s['ErrFileOpen'] + '\n\n' + filename
                    messagebox.showwarning(s['Error'], meldung)
                                
                # Fortschritt in Progressbar anzeigen
                self.status.progress.step(1)
                self.status.progress.update()

            # Speichern
            self.korpus.db_save()

            # Progressbar zurücksetzen
            self.status.progress.config(value = 0)
            self.status.progress.update()
            
            # Dateiliste anzeigen
            self.zeigDateiliste()
        else:
            messagebox.showwarning(s['Error'], s['ErrNoKorp'])


    def webFiles_hinzu(self):
        """URLs aus dem Web zusammensuchen (crawlen) und hinzufügen"""
        if self.korpus != None:
            try:
                anzahl = int(self.webAnzahl.get())
            except:
                anzahl = 10
                
            # Adresse ohne Schema eingegeben?
            url = self.webUrl.get()
            if url[:7] not in ['http://', 'https:/']:
                url = 'http://' + url

            self.busyCursorOn()
            self.status.set(s['StatusWeb'] % anzahl)
            try:
                if anzahl == 1:
                    # File direkt einlesen
                    self.korpus.file_add(filename=url, encoding=self.dateikodierung.get())
                else:
                    # Crawler starten
                    self.korpus.crawlWeb(url=url, anzahl=anzahl, basis=self.webBasis.get() )
                self.zeigDateiliste()
            except Exception as e:
                print('Error:', e)
                self.status.set(s['Error'])
                messagebox.showwarning(s['Error'], s['ErrWeb'])

            # Speichern
            self.korpus.db_save()
            self.status.set(' ')
            self.busyCursorOff()
        else:
            messagebox.showwarning(s['Error'], s['ErrNoKorp'])


    def datei_anzeigen(self, event=""):
        """ Text aus Datenbank anzeigen """
        item_id = self.datliste.focus()
        filename = self.datliste.item(item_id)['values'][0]
        txt = self.korpus.file_get(filename)
        try:
            self.ted.destroy()
        except:
            pass
        self.TextEditDialog(titel=filename, text=txt)


    def datei_speichern(self, event=''):
        """Änderungen an Datei speichern (einfach neu hinzufügen)"""
        self.korpus.file_add(filename=self.editFilename, txt=self.editTxt.get(1.0, END) )
        self.korpus.db_save()
        

    def datei_oeffnen(self, event=''):
        """ Datei mit Standardanwendung oeffnen """
        try:
            item_id = self.datliste.focus()
            pfad = self.datliste.item(item_id)['values'][0]
            try:
                # Windows
                if os.name == "nt":
                    os.startfile(pfad)
                # Macintosh
                elif sys.platform == "darwin":
                    subprocess.call(['open', pfad])
                # Generisches Unix (X11)
                else:
                    subprocess.Popen(['xdg-open', pfad])
            except:
                meldung = s['ErrFileOpen'] + '\n\n' + pfad
                messagebox.showwarning(s['Error'], meldung)
        except:
            messagebox.showwarning(s['Error'], s['ErrSelect'])


    def datei_entfernen(self, event=''):
        item_ids = self.datliste.selection()
        if len(item_ids) > 0:
            self.busyCursorOn()
            d = []
            for i in item_ids:
                f = self.datliste.item(i)['values'][0]
                self.korpus.file_del(filename=f)

            # Speichern
            self.korpus.db_save()           
            self.busyCursorOff()
            self.zeigDateiliste()
        else:
            messagebox.showwarning(s['Error'], s['ErrSelect'])


    def datei_info(self, event=''):
        item_id = self.datliste.focus()
        if item_id != None:
            dateipfad = self.datliste.item(item_id)['values'][0]
            # Anzahl Tokens/Types aus Datenbank holen
            anz_tokens, anz_types = self.korpus.getFileInfo(dateipfad)
            info = s['DateiInfo2'] % (os.path.basename(dateipfad), dateipfad, anz_tokens, anz_types)
            messagebox.showinfo(s['DateiInfo'], info)
        else:
            messagebox.showwarning(s['Error'], s['ErrSelect'])


    def zeigDateiliste(self):
        # erst aufräumen
        for i in self.datliste.get_children():
            self.datliste.delete(i)
        for i in self.freqliste.get_children():
            self.freqliste.delete(i)
        for i in self.konkliste.get_children():
            self.konkliste.delete(i)
        
        gesamtgroesse = 0
        liste = []
        if self.korpus != None:
            liste = self.korpus.getDbFileNames()
            
            for x in liste:
                self.datliste.insert('', 'end', values=(x,) )
            # Gesamtzahl Tokens/Types aus Datenbank holen
            anz_tokens, anz_types = self.korpus.getDbSize()
            if anz_tokens == None: anz_tokens = 0
            if anz_types == None: anz_types = 0
            self.status.indicator(s['StatusFiles'] % ("{:,}".format(len(liste)), "{:,}".format(anz_tokens)))
        else:
            self.status.indicator(s['StatusNoKorp'])
        self.nb.select(0)
        
        self.infotxt.set(s['InfoText'])
    

    def zeigWortfrequenz(self, event=''):
        if self.korpus != None:
            self.busyCursorOn()
            # Liste leer machen
            for i in self.freqliste.get_children():
                self.freqliste.delete(i)

            freq = self.korpus.getWordFreq( sort=self.sortModusFreq.get(),
                    grossklein=self.grosskleinFreq.get(), minFreq=self.minFreq.get(),
                    maxFreq=self.maxFreq.get(), suchFreq=self.suchFreq.get() )
            self.suchFreq.set('')


            # Einstellungen für die Frequenzspalte
            self.freqliste.column('freq', anchor=CENTER)
            if self.sortModusFreq.get() == 'retro':
                self.freqliste.column('wort', anchor=E)
            else:
                self.freqliste.column('wort', anchor=W)
            
            # Resultate einfügen
            for x in freq:
                self.freqliste.insert('', 'end', values=(x[0], x[1]) )

            # Notebook-Tab Formen/Frequenz
            self.nb.select(1)
            
            # Wenn Großschreibung hier ignoriert wird, dann sollte sie auch
            # bei den Konkordanzen ignoriert werden...
            if self.grosskleinFreq.get() == 1:
                self.konkGrossklein.set(1)
            else:
                self.konkGrossklein.set(0) 
            
            self.status.set( s['StatusTreffer'] % len(self.korpus.freqListe) )
            self.busyCursorOff()
        else:
            messagebox.showwarning(s['Error'], s['ErrNoKorp'])


    def zeigFormInfo(self, event=''):
        item_id = self.freqliste.focus()
        item = self.freqliste.item(item_id)['values']
        form = item[0]
        freq = item[1]
        freqfloat = float(freq)
        anzTokens, anzTypes = self.korpus.getDbSize()
        prozent = freqfloat / anzTokens * 100
        txt = s['FTformInfo'] % (anzTokens, form, freq, prozent)
        messagebox.showinfo(form, txt )


    def zeigKonkordanz(self, event=''):
        self.busyCursorOn()
        if self.nb.index("current") == 1:
            self.konkGanzewoerter.set(1)
            item_id = self.freqliste.focus()
            item = self.freqliste.item(item_id)['values']
            such = item[0]
        else:
            such = self.suchwort.get()


        self.status.set(s['StatusSuchKonk'])
        # Liste leeren
        for i in self.konkliste.get_children():
            self.konkliste.delete(i)
        
        konkRes = self.korpus.getWordConc( such=such, ganzWort=self.konkGanzewoerter.get(), 
                    grossklein=self.konkGrossklein.get(),
                    cL=self.konkContextL.get(), cR=self.konkContextR.get(),
                    sort=self.sortModusKonk.get(), markKonk=self.markKonk.get() )
        self.suchwort.delete(0, END)
        self.suchwort.insert(END, such)
        for x in konkRes:
            if self.markKonk.get() == 1:
                konk = x[0] + cfg['konkMarkL'] + x[1] + cfg['konkMarkR'] + x[2]
            else:
                konk = x[0] + x[1] + x[2]
            konk = ' '.join(konk.split())
            self.konkliste.insert('', 'end', values=(konk,) )

        # Notebook Tab Konkordanzen
        self.nb.select(2)
        self.status.set(s['StatusTreffer'] % len(konkRes) )

        self.busyCursorOff()


    def zeigZitat(self, event=''):
        self.zitat.delete(1.0, END)
        item_id = self.konkliste.focus()
        # Der Konkordanz-Eintrag wird über die Reihenfolge bestimmt
        i = self.konkliste.index(item_id)
        cL = int(cfg['zitContextL'])
        cR = int(cfg['zitContextR'])
        
        res = self.korpus.getContext(i, contextL=cL, contextR=cR)
        
        self.zitat.insert(END, '\n')
        self.zitat.insert(END, self.korpus.zitatDateiPfad, ('oeffnen', 'blau'))
        self.zitat.insert(END, s['ZToeffnen2'])
        self.zitat.insert(END, '\n- - - - -\n\n')
        self.zitat.insert(END, res[0])
        self.zitat.insert(END, res[1], 'rot')
        self.zitat.insert(END, res[2])
        # Notebook Tab Zitat
        self.nb.select(3)


    def zitatDatei_oeffnen(self, event=''):
        try:
            pfad = self.korpus.zitatDateiPfad
            # pfad = pfad.encode(sys.getfilesystemencoding())
            # Windows
            if os.name == "nt":
                os.startfile(pfad)
            # Macintosh
            elif sys.platform == "darwin":
                subprocess.call(['open', pfad])
            # Generisches Unix (X11)
            else:
                subprocess.Popen(['xdg-open', pfad])

        except:
            meldung = s['ErrFileOpen'] + '\n\n' + pfad
            messagebox.showwarning(s['Error'], meldung)



##################################
### Allerlei Hilfsfunktionen   ###
##################################


### Recent Files ###

    def add_recent(self,filename=''):
        if filename != '':
            filename = os.path.normpath(filename)
            # Wenn schon in Liste vorhanden, dann erst löschen, 
            # anschließend am Anfang hinzufügen
            self.remove_recent(filename)                
            cfg['LetzteKorpora'].insert(0, filename)
            self.update_recentfiles_menu()

    def remove_recent(self,filename=''):
        filename = os.path.normpath(filename)
        # Item aus Liste löschen (wenn vorhanden), Liste kürzen 
        if filename in cfg['LetzteKorpora']:
            cfg['LetzteKorpora'].remove(filename)
        if len(cfg['LetzteKorpora']) > 5:
            cfg['LetzteKorpora'].pop()
        self.update_recentfiles_menu()

    def update_recentfiles_menu(self):
        # Menü erst leeren, dann wieder füllen
        self.recentfilesmenu.delete(0,5)
        for f in cfg['LetzteKorpora']:
            self.recentfilesmenu.add_command(label=f, underline=0, command = self.__korpus_open(f) )
            
    # siehe für diese 'closure' auch: 
    # http://stackoverflow.com/questions/21963373/tkinter-how-to-open-recent-files
    def __korpus_open(self,f):
        def korpus_open_caller(f=f):
            self.korpus_open(f)
        return korpus_open_caller


### Export-Funktionen ###

    def exportFreq2excel(self):
        wortliste = self.korpus.freqListe
        try:
            if len(wortliste) > 1:
                meinXL = Dispatch("Excel.Application")
                meinXL.Visible = 1
                meinXL.Workbooks.Add()
                tabelle = meinXL.Workbooks(1).Sheets(1)
                tabelle.Cells(2,1).Value = 'ACHTUNG - ATTENTION'
                tabelle.Cells(3,1).Value = 'Dies kann dauern...'
                tabelle.Cells(4,1).Value = 'This can take a while...'
                titel = '%(Titel)s %(Version)s' % s
                tabelle.Cells(2,4).Value = titel
                tabelle.Cells(3,4).Value = s['Copyright']
                tabelle.Cells(4,4).Value = s['ProgURL']
                tabelle.Range(tabelle.Cells(1,1), tabelle.Cells(len(wortliste),2)).Value=wortliste
            else:
                messagebox.showwarning(s['ErrExport'], s['ErrNoFreqList'])
        except:
            messagebox.showwarning(s['ErrExport'], s['ErrExcel'])


    def exportFreq2csv(self):         
        f = filedialog.asksaveasfilename(defaultextension='.csv', initialdir=cfg['KorpDir'])
        with io.open(f, mode="w", encoding="utf-8") as d:
            for x in self.korpus.freqListe:
                zeile = x[0] + '\t' + str(x[1]) + '\n'
                d.write(zeile)

        messagebox.showinfo('Export', s['StatusExpFreq'])


    def exportConc2csv(self):
        konkliste = []
        for x in self.korpus.concListe:
            # Die drei Originalen Konkordanzelemente
            # man könnte hier den eigentlichen Suchbegriff (in der Mitte) auch noch markieren
            konk = x[0] + cfg['konkMarkL'] + x[1] + cfg['konkMarkR'] + x[2]
            # Eventuelle \n entfernen
            konk = ' '.join(konk.split())
            konkliste.append([konk, x[3]])
            
        f = filedialog.asksaveasfilename(defaultextension='.csv', initialdir=cfg['KorpDir'])
        with io.open(f, mode="w", encoding="utf-8") as d:
            for x in konkliste:
                zeile = '\t'.join(x) + '\n'
                d.write(zeile)
                
        messagebox.showinfo('Export', s['StatusExpKonk'])


    def exportConc2txt(self):
        konkliste = []
        for x in self.korpus.concListe:
            # Die drei Originalen Konkordanzelemente
            # man könnte hier den eigentlichen Suchbegriff (in der Mitte) auch noch markieren
            konk = x[0] + cfg['konkMarkL'] + x[1] + cfg['konkMarkR'] + x[2]
            # Eventuelle \n entfernen
            konk = ' '.join(konk.split())
            konkliste.append(konk)
            
        inhalt = "%(Titel)s %(Version)s\n%(Copyright)s\n(%(ExportHinweis)s)\n\n" % s
        inhalt = inhalt + '\n'.join(konkliste)
        f = filedialog.asksaveasfilename(defaultextension='.txt', initialdir=cfg['KorpDir'])
        with io.open(f, mode="w", encoding="utf-8") as d:
            d.write(inhalt)

        messagebox.showinfo('Export', s['StatusExpKonk'])


    def exportConc2excel(self):
        konkliste = []
        for x in self.korpus.concListe:
            a = ' '.join(x[0].split())
            b = ' '.join(x[1].split())
            c = ' '.join(x[2].split())
            pfad = x[3]
            konkliste.append([a,b,c,pfad])
        try:
            if len(konkliste) > 1:
                meinXL = Dispatch("Excel.Application")
                meinXL.Visible = 1
                meinXL.Workbooks.Add()
                tabelle = meinXL.Workbooks(1).Sheets(1)
                tabelle.Range(tabelle.Cells(1,1), tabelle.Cells(len(konkliste),4)).Value=konkliste
            else:
                messagebox.showwarning(s['ErrExport'], s['ErrNoKonkList'])
        except:
            messagebox.showwarning(s['ErrExport'], s['ErrExcel']) 
 

### Andere Hilfsfunktionen ###

    def optionen_speichern(self):
        try:
            cfg['Benutzersprache'] = self.sprache_aendern.get()
            cfg['fenster'] = self.root.geometry()
            cfg['freqMin'] = self.minFreq.get()
            cfg['freqMax'] = self.maxFreq.get()
            cfg['konkContextL'] = self.konkContextL.get()
            cfg['konkContextR'] = self.konkContextR.get()
            cfg['FileKodierung'] = self.dateikodierung.get()

            with open(cfgFile, 'w') as outfile:
                json.dump(cfg, outfile, indent=2) 
        except:
            messagebox.showwarning(s['Error'], s['ErrOptSave'])


    def copy2clipboard(self):
        self.root.clipboard_clear()
        # im Zitat-Tab 
        if self.nb.index("current") == 3:
            try:
                txt = self.zitat.get(SEL_FIRST, SEL_LAST)
            except TclError:
                messagebox.showwarning(s['Error'], s['ErrSelect'])
        # in der Konkordanzliste 
        elif self.nb.index("current") == 2:
            item_ids = self.konkliste.selection()
            txt = []
            for i in item_ids:
                txt.append( self.konkliste.item(i)['values'][0] )
            txt = '\n'.join(txt)
        # in der Frequenzliste 
        elif self.nb.index("current") == 1:
            item_ids = self.freqliste.selection()
            txt = []
            for i in item_ids:
                a,b = self.freqliste.item(i)['values']
                t = '%s - %s' % (a,b)
                txt.append( t )
            txt = '\n'.join(txt)            
        # in der Dateiliste 
        elif self.nb.index("current") == 0:
            item_ids = self.datliste.selection()
            txt = []
            for i in item_ids:
                txt.append( self.datliste.item(i)['values'][0] )
            txt = '\n'.join(txt)

        self.root.clipboard_append(txt)
        self.status.set(s['StatusClip'])
        # messagebox.showinfo('', txt)


    def busyCursorOn(self):
        # Cursor auf 'busy/wait'
        self.root.config(cursor='watch')
        self.root.update_idletasks()
        self.root.update()

    def busyCursorOff(self):
        # Cursor auf 'normal'
        self.root.config(cursor='')
        self.root.update_idletasks()
        self.root.update()


    def statusAktualisieren(self, event=''):
        if self.korpus != None:
            # Frequenzliste ausgewählt
            if self.nb.index("current") == 1:
                self.status.set( s['StatusTreffer'] % len(self.korpus.freqListe) )
            # Konkordanzliste ausgewählt
            if self.nb.index("current") == 2:
                self.status.set( s['StatusTreffer'] % len(self.korpus.concListe) )
            # Dateiliste ausgewählt
            if self.nb.index("current") == 0:
                self.status.set( s['StatusKorpDat'] % (self.korpus.getDbName(), self.dateikodierung.get()) )
        else:
            self.status.set( s['Willkommen'] )


    def sprachwarnung(self):
        self.optionen_speichern()
        messagebox.showwarning(s['SpracheAendern'], s['SpracheInfo'])


    def about(self):
        info = "%(Titel)s %(Version)s\n(%(VersionDatum)s)\n\n%(Copyright)s\n%(eMail)s\n\n%(ProgURL)s  \n\n%(ProgInfo)s" % s
        messagebox.showinfo('TextSTAT', info)


    def regex_hilfe(self):
        try:
            webbrowser.open("http://regexone.com/")
        except:
            self.warnung(titel=s['Error'], text=s['ErrFileOpen'])


    def ende(self):
        if self.korpus != None:
            self.korpus.db_close()
        self.optionen_speichern()
        self.root.destroy()
        self.root.quit()



if __name__ == '__main__':

    app = GUI(root)
    root.mainloop()
